@@ -1,6 +1,6 @@
-/* sha3-224-meta.c
+/* sha512-256-meta.c
 
-   Copyright (C) 2012 Niels Möller
+   Copyright (C) 2014 Niels Möller
 
    This file is part of GNU Nettle.
 
@@ -35,7 +35,15 @@
 
 #include "nettle-meta.h"
 
-#include "sha3.h"
+#include "sha2.h"
+
+const struct nettle_hash nettle_sha512_256 =
+  {
+    "sha512-256", sizeof(struct sha512_ctx),
+    SHA512_256_DIGEST_SIZE,
+    SHA512_256_BLOCK_SIZE,
+    (nettle_hash_init_func *) sha512_256_init,
+    (nettle_hash_update_func *) sha512_256_update,
+    (nettle_hash_digest_func *) sha512_256_digest
+  };
 
-const struct nettle_hash nettle_sha3_224
-= _NETTLE_HASH(sha3_224, SHA3_224);
