@@ -19,6 +19,15 @@ ref_mod (mp_limb_t *rp, const mp_limb_t *ap, const mp_limb_t *mp, mp_size_t mn)
 #define MAX_SIZE (2*MAX_ECC_SIZE)
 #define COUNT 50000
 
+/* Destructively normalize tp, then compare */
+static int
+mod_equal(const struct ecc_modulo *m, const mp_limb_t *ref, mp_limb_t *tp)
+{
+  if (mpn_cmp (tp, m->m, m->size) >= 0)
+    mpn_sub_n (tp, tp, m->m, m->size);
+  return mpn_cmp (ref, tp, m->size) == 0;
+}
+
 static void
 test_one(const char *name,
 	 const struct ecc_modulo *m,
@@ -33,13 +42,10 @@ test_one(const char *name,
   ref_mod (ref, a, m->m, m->size);
 
   mpn_copyi (t, a, 2*m->size);
-  m->mod (m, t);
-  if (mpn_cmp (t, m->m, m->size) >= 0)
-    mpn_sub_n (t, t, m->m, m->size);
-
-  if (mpn_cmp (t, ref, m->size))
+  m->mod (m, t, t);
+  if (!mod_equal (m, ref, t))
     {
-      fprintf (stderr, "m->mod %s failed: bit_size = %u\n",
+      fprintf (stderr, "m->mod %s failed: bit_size = %u, rp == xp\n",
 	       name, m->bit_size);
 
       fprintf (stderr, "a   = ");
@@ -52,16 +58,30 @@ test_one(const char *name,
       abort ();
     }
 
+  mpn_copyi (t, a, 2*m->size);
+  m->mod (m, t + m->size, t);
+  if (!mod_equal (m, ref, t + m->size))
+    {
+      fprintf (stderr, "m->mod %s failed: bit_size = %u, rp == xp + size\n",
+	       name, m->bit_size);
+
+      fprintf (stderr, "a   = ");
+      mpn_out_str (stderr, 16, a, 2*m->size);
+      fprintf (stderr, "\nt   = ");
+      mpn_out_str (stderr, 16, t + m->size, m->size);
+      fprintf (stderr, " (bad)\nref = ");
+      mpn_out_str (stderr, 16, ref, m->size);
+      fprintf (stderr, "\n");
+      abort ();
+    }
+
   if (m->B_size < m->size)
     {
       mpn_copyi (t, a, 2*m->size);
-      ecc_mod (m, t);
-      if (mpn_cmp (t, m->m, m->size) >= 0)
-	mpn_sub_n (t, t, m->m, m->size);
-
-      if (mpn_cmp (t, ref, m->size))
+      ecc_mod (m, t, t);
+      if (!mod_equal (m, ref, t))
 	{
-	  fprintf (stderr, "ecc_mod %s failed: bit_size = %u\n",
+	  fprintf (stderr, "ecc_mod %s failed: bit_size = %u, rp == xp\n",
 		   name, m->bit_size);
 	  fprintf (stderr, "a   = ");
 	  mpn_out_str (stderr, 16, a, 2*m->size);
@@ -72,6 +92,22 @@ test_one(const char *name,
 	  fprintf (stderr, "\n");
 	  abort ();
 	}
+
+      mpn_copyi (t, a, 2*m->size);
+      ecc_mod (m, t + m->size, t);
+      if (!mod_equal (m, ref, t + m->size))
+	{
+	  fprintf (stderr, "ecc_mod %s failed: bit_size = %u, rp == xp + size\n",
+		   name, m->bit_size);
+	  fprintf (stderr, "a   = ");
+	  mpn_out_str (stderr, 16, a, 2*m->size);
+	  fprintf (stderr, "\nt   = ");
+	  mpn_out_str (stderr, 16, t + m->size, m->size);
+	  fprintf (stderr, " (bad)\nref = ");
+	  mpn_out_str (stderr, 16, ref, m->size);
+	  fprintf (stderr, "\n");
+	  abort ();
+	}
     }
 }
 
@@ -105,23 +141,23 @@ test_fixed (void)
   /* Triggered a bug reported by Hanno Böck. */
   mpz_set_str (r, "FFFFFFFF00000001000000000000000000000000FFFFFFFFFFFFFFFF001C2C00", 16);
   mpz_mul_2exp (r, r, 256);
-  test_one ("p", &nettle_secp_256r1.p, r);
-  test_one ("q", &nettle_secp_256r1.q, r);
+  test_one ("p", &_nettle_secp_256r1.p, r);
+  test_one ("q", &_nettle_secp_256r1.q, r);
 
   mpz_set_str (r, "ffffffff00000001fffffffeffffffffffffffffffffffffffffffc0000000000007ffffffffffffffffffffffffffff00000000000000000fffffffffffffff", 16);
-  test_one ("p", &nettle_secp_256r1.p, r);
-  test_one ("q", &nettle_secp_256r1.q, r);
+  test_one ("p", &_nettle_secp_256r1.p, r);
+  test_one ("q", &_nettle_secp_256r1.q, r);
 
   /* Triggered a bug reported by Hanno Böck. */
   mpz_set_str (r, "4c9000000000000000000000000000000000000000000000004a604db486e000000000000000000000000000000000000000121025be29575adb2c8ffffffffffffffffffffffffffffffffffffffffffffffffffffffff", 16);
-  test_one ("p", &nettle_secp_384r1.p, r);
-  test_one ("q", &nettle_secp_384r1.q, r);
+  test_one ("p", &_nettle_secp_384r1.p, r);
+  test_one ("q", &_nettle_secp_384r1.q, r);
 
   /* Triggered a carry bug in development version. */
   mpz_set_str (r, "e64a84643150260640e4677c19ffc4faef06042132b86af6e9ee33fe1850222e57a514d5f1d6d444008bb896a96a43d5629945e57548f5e12f66be132b24110cbb2df6d7d3dd3aaadc98b0bbf29573843ad72e57f59fc5d4f56cc599da18bb99", 16);
 
-  test_one ("p", &nettle_secp_384r1.p, r);
-  test_one ("q", &nettle_secp_384r1.q, r);
+  test_one ("p", &_nettle_secp_384r1.p, r);
+  test_one ("q", &_nettle_secp_384r1.q, r);
 
   mpz_clear (r);
 }
