@@ -14,42 +14,34 @@ test_main (void)
     {
       const struct ecc_curve *ecc = ecc_curves[i];
       mp_size_t size = ecc_size (ecc);
+      mp_limb_t *g = xalloc_limbs (ecc_size_a (ecc));
       mp_limb_t *p = xalloc_limbs (ecc_size_j (ecc));
       mp_limb_t *q = xalloc_limbs (ecc_size_j (ecc));
       mp_limb_t *n = xalloc_limbs (size);
       mp_limb_t *scratch = xalloc_limbs (ecc->mul_itch);
       unsigned j;
       
+      test_ecc_get_ga (i, g);
       mpn_zero (n, size);
 
-      n[0] = 1;
-      ecc->mul (ecc, p, n, ecc->g, scratch);
-      ecc->h_to_a (ecc, 0, p, p, scratch);
-
-      if (mpn_cmp (p, ecc->g, 2*size) != 0)
-	die ("curve %d: ecc->mul with n = 1 failed.\n", ecc->p.bit_size);
-
-      for (n[0] = 2; n[0] <= 4; n[0]++)
+      for (n[0] = 1; n[0] <= 4; n[0]++)
 	{
-	  ecc->mul (ecc, p, n, ecc->g, scratch);
+	  ecc->mul (ecc, p, n, g, scratch);
 	  test_ecc_mul_h (i, n[0], p);
 	}
 
       /* (order - 1) * g = - g */
       mpn_sub_1 (n, ecc->q.m, size, 1);
-      ecc->mul (ecc, p, n, ecc->g, scratch);
+      ecc->mul (ecc, p, n, g, scratch);
       ecc->h_to_a (ecc, 0, p, p, scratch);
-      if (ecc->p.bit_size == 255)
+      if (ecc->p.bit_size == 255 || ecc->p.bit_size == 448)
 	/* For edwards curves, - (x,y ) == (-x, y). FIXME: Swap x and
 	   y, to get identical negation? */
 	mpn_sub_n (p, ecc->p.m, p, size);
       else
 	mpn_sub_n (p + size, ecc->p.m, p + size, size);
-      if (mpn_cmp (p, ecc->g, 2*size) != 0)
-	{
-	  fprintf (stderr, "ecc->mul with n = order - 1 failed.\n");
-	  abort ();
-	}
+
+      test_ecc_ga (i, p);
 
       mpn_zero (n, size);
 
@@ -64,7 +56,7 @@ test_main (void)
 	  mpz_limbs_copy (n, r, size);
 	  n[size - 1] %= ecc->q.m[size - 1];
 
-	  ecc->mul (ecc, p, n, ecc->g, scratch);
+	  ecc->mul (ecc, p, n, g, scratch);
 	  ecc->h_to_a (ecc, 0, p, p, scratch);
 
 	  ecc->mul_g (ecc, q, n, scratch);
@@ -92,6 +84,7 @@ test_main (void)
 	      abort ();
 	    }
 	}
+      free (g);
       free (n);
       free (p);
       free (q);
